##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
# Modified from CMF's Favorite.py by Kelvin Mo 2002
# 
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
# 
##############################################################################
"""
    Bookmarks are pointers to resources within a ZEverything site.
"""
__version__ = "0.5"

import Globals
from Globals import HTMLFile, HTML
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore.utils import getToolByName
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.WorkflowCore import WorkflowAction
import string, urlparse

factory_type_information = ( { 'id'             : 'Bookmark'
                             , 'meta_type'      : 'Bookmark'
                             , 'description'    : """\
A bookmark to something on this site."""
                             , 'icon'           : 'link_icon.gif'
                             , 'product'        : '' # Hidden
                             , 'factory'        : 'addBookmark'
                             , 'immediate_view' : 'bookmark_goto'
                             , 'actions'        :
                                ( { 'id'            : 'view'
                                  , 'name'          : 'View'
                                  , 'action'        : 'bookmark_goto'
                                  , 'permissions'   : (
                                      CMFCorePermissions.View, )
                                  },
                                )
                             }
                           ,
                           )

def addBookmark(self, id, title='', remote_url='', description=''):
    """
    Add a bookmark
    """
    portal_url = getToolByName(self, 'portal_url')
    portal_obj = portal_url.getPortalObject()
    content_obj = portal_obj.restrictedTraverse( remote_url )
    relUrl = portal_url.getRelativeUrl( content_obj )
    o=Bookmark( id, title, relUrl, description )
    self._setObject(id,o)


class Bookmark(PortalContent):
    """
        Bookmark to something on this site
    """

    meta_type='Bookmark'
    portal_type='Bookmark'

    def __init__(self, id, title='', remote_url='', description=''):
        self.id=id
        self.title=title
        self.remote_url=remote_url
        self.description = description

    def getRemoteUrl(self):
        """
            Returns the remote URL of the Link
        """
        portal_url = getToolByName(self, 'portal_url')
        if self.remote_url:
            return portal_url() + '/' + self.remote_url
        else:
            return portal_url()

    def getIcon(self, relative_to_portal=0):
        """
        Display an icon based on what the bookmark links to.
        """
        try:
            return self.getObject().getIcon(relative_to_portal)
        except:
            return 'p_/broken'

    def getObject(self):
        """
        Return the actual object that the bookmark is 
        linking to
        """
        portal_url = getToolByName(self, 'portal_url')
        return portal_url.getPortalObject().restrictedTraverse(self.remote_url)

    def edit(self, remote_url):
        """
        Edit the bookmark.
        """
        # strip off scheme and machine from URL if present
        tokens = urlparse.urlparse(remote_url, 'http')
        if tokens[0] or tokens[1]:
            t=('', '') + tokens[2:]
            remote_url=urlparse.urlunparse(t)
        # if URL begins with site URL, remove site URL
        portal_url = getToolByName(self, 'portal_url').getPortalPath()
        i=string.find(remote_url, portal_url)
        if i==0:
            remote_url=remote_url[len(portal_url):]
        # if site is still absolute, make it relative
        if remote_url[0]=='/':
            remote_url=remote_url[1:]
        self.remote_url=remote_url

    edit=WorkflowAction(edit)

Globals.default__class_init__(Bookmark)

