##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
# Modified 2002 Kelvin Mo
# 
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
# 
##############################################################################


"""
  This membership tool organises members, their levels and voting rights.
"""
__version__='0.5'


from Products.CMFCore.utils import _getAuthenticatedUser, _checkPermission
from Products.CMFCore.ActionsTool import ActionInformation
import Products.CMFCore.MembershipTool
from UserNode import addUserNode

from Globals import InitializeClass, DTMLFile, PersistentMapping
from AccessControl import ClassSecurityInfo
from Products.CMFCore.Expression import Expression
from Products.CMFCore.CMFCorePermissions import View, AccessContentsInformation
from Products.CMFCore.CMFCorePermissions import ListPortalMembers, AddPortalMember
from Products.CMFCore.CMFCorePermissions import ManagePortal
from utils import _dtmldir
from OFS.SimpleItem import SimpleItem

class LevelProperties(SimpleItem):
       
    id = 0
    entry_xp = 0
    entry_writeup = 0
    daily_vote = 0
    daily_c = 0
    powers = []

    def __init__(self, id, title, entry_xp, entry_writeup, daily_vote, daily_c, powers=[]):
        self.id = id
        self.title = title
        self.entry_xp = entry_xp
        self.entry_writeup = entry_writeup
        self.daily_vote = daily_vote
        self.daily_c = daily_c
        self.powers = powers

InitializeClass(LevelProperties)

DEFAULT_LEVELS = [LevelProperties(0, "This shouldn't happen", 0, 0, 0, 0),
		  LevelProperties(1, 'Level 1', 0, 0, 0, 0),
		  LevelProperties(2, 'Level 2', 50, 25, 10, 0),
		  LevelProperties(3, 'Level 3', 200, 70, 20, 0),
		  LevelProperties(4, 'Level 4', 400, 150, 30, 1),
		  LevelProperties(5, 'Level 5', 800, 250, 45, 1),
		  LevelProperties(6, 'Level 6', 1350, 380, 60, 2),
		  LevelProperties(7, 'Level 7', 2100, 515, 75, 2),
		  LevelProperties(8, 'Level 8', 2900, 700, 90, 3),
		  LevelProperties(9, 'Level 9', 4000, 900, 105, 3),
		  LevelProperties(10, 'Level 10', 7500, 1215, 125, 4),
		  LevelProperties(11, 'Level 11', 13000, 1800, 150, 4),
		  LevelProperties(12, 'Level 12', 23000, 2700, 200, 5),
		  LevelProperties(13, 'Level 13', 38000, 4500, 300, 5)]

class MembershipTool (Products.CMFCore.MembershipTool.MembershipTool):
    """
    """
    _actions =[ActionInformation(id='login'
                            , title='Login'
                            , description='Click here to Login'
                            , action=Expression(text='string: ${portal_url}/login_form')
                            , permissions=(View,)
                            , category='user'
                            , condition=Expression(text='not: member')
                            , visible=1
                             ) 
          , ActionInformation(id='preferences'
                            , title='Preferences'
                            , description='Change your user preferences'
                            , action=Expression(text='string: ${portal_url}/personalize_form')
                            , permissions=(View,)
                            , category='user'
                            , condition=Expression(text='member')
                            , visible=1
                             )
          , ActionInformation(id='logout'
                            , title='Log out'
                            , description='Click here to logout'
                            , action=Expression(text='string: ${portal_url}/logout')
                            , permissions=(View,)
                            , category='user'
                            , condition=Expression(text='member')
                            , visible=1
                             )
          , ActionInformation(id='addFavorite'
                            , title='Bookmark'
                            , description='Add this item to your favorites'
                            , action=Expression(text='string: ${request/URL0}/add_bookmark')
                            , permissions=(View,)
                            , category='user'
                            , condition=Expression(text='python: portal.portal_membership.getHomeFolder()')
                            , visible=1
                             )
          , ActionInformation(id='mystuff'
                            , title='Myself'
                            , description='Goto your home node'
                            , action=Expression(text='python: portal.portal_membership.getHomeUrl()')
                            , permissions=(View,)
                            , category='user'
                            , condition=Expression(text='python: member and portal.portal_membership.getHomeFolder()')
                            , visible=1
                             )
          , ActionInformation(id='scratchpad'
                            , title='Scratchpad'
                            , description='View your scratchpad'
                            , action=Expression(text='python: portal.portal_membership.getHomeUrl() + \'/scratchpad\'')
                            , permissions=(View,)
                            , category='user'
                            , condition=Expression(text='python: portal.portal_membership.getHomeUrl()')
                            , visible=1
                             )
          , ActionInformation(id='favorites'
                            , title='My Bookmarks'
                            , description='Browser your bookmarks'
                            , action=Expression(text='python: portal.portal_membership.getHomeUrl() + \'/storage/bookmarks/folder_contents\'')
                            , permissions=(View,)
                            , category='user'
                            , condition=Expression(text='python: member and hasattr(portal.portal_membership.getHomeFolder().getStorage(), \'bookmarks\')')
                            , visible=1
                             )
          , ActionInformation(id='random_node'
                            , title='Random Node'
                            , description='Go to a random node'
                            , action=Expression(text='string: random_node')
                            , permissions=(View,)
                            , category='user'
                            , visible=1
                             )
          , ActionInformation(id='create_node'
                            , title='Create Node'
                            , description='Create a new node'
                            , action=Expression(text='string: node_create_form')
                            , permissions=(View,)
                            , category='user'
                            , visible=1
                            , condition=Expression(text='member')
                             )
                           ]


    meta_type = 'ZEverything Membership Tool'

    manage_options = (Products.CMFCore.MembershipTool.MembershipTool.manage_options +
                      ({'label': 'Levels',
                        'action': 'manage_levelsForm'}, )
                      )
    levels = []

    security = ClassSecurityInfo()
    
    def __init__(self, levels = DEFAULT_LEVELS):
        self.levels = DEFAULT_LEVELS

    #
    #   ZMI methods
    #
    security.declareProtected(ManagePortal, 'manage_overview')
    manage_overview = DTMLFile('explainMembershipTool', _dtmldir)

    security.declareProtected(ManagePortal, 'manage_levelsForm')
    manage_levelsForm = DTMLFile('levels', _dtmldir)

    security.declareProtected(ManagePortal, 'manage_levelDetailForm')
    manage_levelDetailForm = DTMLFile('levelDetail', _dtmldir)

    security.declareProtected(ManagePortal, 'manage_levels')
    def manage_levels(self, REQUEST):
        """ Changes the properties of each of the levels. """
        for i in self.levels:
            id = i.id
            for j in ['title', 'entry_xp', 'entry_writeup', 'daily_vote', 'daily_c']:
                attr = '%s_%s' % (str(id), j)
                if REQUEST.has_key(attr):
                    setattr(i, attr, REQUEST.get(attr))
            if REQUEST.has_key('%s_powers' % str(id)):
                i.powers = REQUEST.get('%s_powers' % str(id)).split()
        if REQUEST is not None:
            return self.manage_levelsForm(self, REQUEST, manage_tabs_message='Levels changed.')

    #
    #   'portal_membership' interface methods
    #
    security.declareProtected(ListPortalMembers, 'getRoster')
    def getRoster(self):
        '''
        Returns a list of user names
        '''
        isManager = _checkPermission('Manage portal', self)
        roster = []
        for member in self.listMembers():
            if isManager or member.listed:
                roster.append({'id':member.getUserName(),
                               'listed':member.listed})
        return roster

    def addMember(self, id, password, roles, domains, properties=None):
        '''Adds a new member to the user folder.  Security checks will have
        already been performed.  Called by portal_registration.
        '''
        Products.CMFCore.MembershipTool.MembershipTool.addMember(self, id, password, roles, domains, properties)
        self.createMemberarea(id)


    security.declareProtected(ManagePortal, 'createMemberarea')
    def createMemberarea(self, member_id):
        """
        Create a member area
        """
        parent = self.aq_inner.aq_parent
        members = getattr(parent, 'members', None)

        if members is not None and not hasattr(members, member_id):
            addUserNode(members, member_id)
            f=getattr(members, member_id)
 
            # Grant ownership to Member
            acl_users = self.__getPUS()
            user = acl_users.getUser(member_id).__of__(acl_users)
            f.changeOwnership(user)
            f.manage_setLocalRoles(member_id, ['Owner'])

    def getHomeFolder(self, id=None, verifyPermission=0):
        """Returns a member's user node object."""
        if id is None:
            member = self.getAuthenticatedMember()
            if not hasattr(member, 'getMemberId'):
                return None
            id = member.getMemberId()
        if hasattr(self, 'members'):
            try:
                folder = self.members[id]
                if verifyPermission and not _checkPermission('View', folder):
                    # Don't return the folder if the user can't get to it.
                    return None
                return folder
            except KeyError: pass
        return None
        
    def getHomeUrl(self, id=None, verifyPermission=0):
        """Returns the URL to a member's user node."""
        home = self.getHomeFolder(id, verifyPermission)
        if home is not None:
            return home.absolute_url()
        else:
            return None

    security.declarePublic('getMemberById')
    def getMemberById(self, id):
        u = self.acl_users.getUser(id)
        if u is not None:
            u = self.wrapUser(u)
        return u

    security.declarePublic('canVote')
    def canVote(self, here, id = None):
        if id is None:
            m = self.getAuthenticatedMember()
        else:
            m = self.getMemberById(id)
        # Can't vote for your own nodes
        if hasattr(here, 'creator'):
            if (here.creator == m.getUserName()):
                return 0
        current_vote = m.getProperty('current_vote', 0)
        current_c = m.getProperty('current_c', 0)
        return ((current_vote > 0) or (current_c > 0))
            

    security.declarePrivate('listActions')
    def listActions(self, info=None):
        '''Lists actions available through the tool.'''
        return self._actions

    # Level policies
    security.declarePublic('incrementXP')
    def incrementXP(self, increment, id=None):
        if increment == 0:
            return 0
        if id is None:
            user = self.getAuthenticatedMember()
        else:
            user = self.getMemberById(id)
        user.setMemberProperties({'xp': user.getProperty('xp', 0) + increment})
        self.levelPrerequisitesChanged(user)
    
    security.declarePrivate('useableLevels')
    def useableLevels(self):
        return self.levels[1:]

    security.declarePublic('getLevel')
    def getLevel(self, level):
        """Returns the LevelProperties object"""
        return self.levels[level]

    security.declarePublic('hasPower')
    def hasPower(self, power, id=None):
        if id is None:
            user = self.getAuthenticatedMember()
        else:
            user = self.getMemberById(id)
        l = user.level
        for i in self.levels[0:l + 1]:
            if power in i.powers:
                return 1
        return 0
    
    security.declarePrivate('levelPrerequisitesChanged')
    def levelPrerequisitesChanged(self, user=None):
        if user is None:
            user = self.getAuthenticatedMember()
        xp = user.getProperty('xp')
        wu = self.countWriteups(user.getUserName())
        criteria = list(self.useableLevels())[:]
        criteria.reverse()
        for level in criteria:
            if (xp >= level.entry_xp) and (wu >= level.entry_writeup):
                user.setMemberProperties({'level': level.id,
                                          'daily_vote': level.daily_vote,
                                          'daily_c': level.daily_c,
                                          'current_vote': level.daily_vote - user.getProperty('current_vote'),
                                          'current_c': level.daily_c - user.getProperty('current_c')})
                return level.id
        return 0

    security.declareProtected(ManagePortal, 'dailyReset')
    def dailyReset(self, user):
        daily_vote = user.getProperty('daily_vote', 0)
        daily_c = user.getProperty('daily_c', 0)
        user.setMemberProperties({'current_vote': daily_vote,
                               'current_c': daily_c})
    
InitializeClass(MembershipTool)
