##############################################################################
#
# Copyright (c) 2002 Kelvin Mo
# 
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
# 
##############################################################################

"""
The node tool holds miscellaneous functions.
"""
__version__='0.01'

from AccessControl import ClassSecurityInfo
from OFS.SimpleItem import SimpleItem
from Products.CMFCore.utils import UniqueObject, getToolByName
from Products.CMFCore import CMFCorePermissions
from utils import _dtmldir

from Globals import InitializeClass, DTMLFile

from DateTime import DateTime

from zehtml.ZEHTMLParser import zehtml_format
from ZENode import addZENode

import random

class NodeTool (UniqueObject, SimpleItem):
    id = 'portal_node'
    meta_type = 'ZEverything Node Tool'
    security = ClassSecurityInfo()

    random_node_cache = []
    random_node_cache_time = 0

    manage_options = (({'label' : 'Overview', 'action' : 'manage_overview'},)
                       + SimpleItem.manage_options)

    #
    #   ZMI methods
    #
    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_overview')
    manage_overview = DTMLFile('explainNodeTool', _dtmldir)

    security.declarePublic('cook_zehtml')
    def cook_zehtml(self, zehtml, level=1):
        return zehtml_format(zehtml, level)

    security.declarePublic('getRandomNodes')
    def getRandomNodes(self, n):
        if ((len(self.random_node_cache) == n) & (DateTime().timeTime() - self.random_node_cache_time < 5*60)):
            return self.random_node_cache
        cat = getToolByName(self, 'portal_catalog')
        nodes = cat(Type = 'Node')
        rand = []
        for i in range(n):
            r = random.choice(nodes)
            rand.append({'url' : r.getURL(), 'title' : r['Title']})
        self.random_node_cache = rand
        self.random_node_cache_time = DateTime().timeTime()
        return rand

    security.declarePublic('getANodeId')
    def getANodeId(self, p):
        prefix = int(DateTime().timeTime())
        suffix = random.randrange(0, 9999)
        while getattr(p, '%d-%04d' % (prefix, suffix), None ) is not None:
            suffix = random.randrange(0, 9999)
        id = '%d-%04d' % (prefix, suffix)
        return id

    security.declarePublic('addNode')
    def addNode(self, id, title, REQUEST=None):
        ''' Adds a node '''
        portal = getToolByName(self, 'portal_url').getPortalObject()
        return addZENode(portal.node, id, title)

    security.declarePublic('writeupSubjects')
    def writeupSubjects(self):
        metadata_tool=getToolByName(self, 'portal_metadata')
        return metadata_tool.getElementSpec('Subject').getPolicy('Writeup').allowedVocabulary()
    
InitializeClass(NodeTool)

