##############################################################################
#
# Copyright (c) 2002 Kelvin Mo
# 
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
# 
##############################################################################

"""ZEverything class
"""
__version__='0.5'
 
import Globals
from Products.CMFCore.PortalObject import PortalObjectBase
from Products.CMFCore import PortalFolder
from Products.CMFCore.TypesTool import ContentFactoryMetadata
from Products.CMFCore.utils import getToolByName
from Products.CMFTopic import Topic, topic_globals
from DublinCore import DefaultDublinCoreImpl

import ZENode, UserNode, Bookmark, MessageTool
import Writeup

import os

factory_type_information = ( ZENode.factory_type_information
                           + UserNode.factory_type_information
                           + Bookmark.factory_type_information
                           + Writeup.factory_type_information
                           + MessageTool.factory_type_information
                           )

DEFAULT_SUBJECT = 'Person'
ALLOWED_SUBJECTS = ('Person', 'Place', 'Idea', 'Thing', 'Definition')


class ZEverythingSite (PortalObjectBase, DefaultDublinCoreImpl):
    """
        The *only* function this class should have is to help in the setup
        of a new ZEverythingSite.  It should not assist in the functionality at all.
    """
    meta_type = 'ZEverything Site'

    _properties = (
        {'id':'title', 'type':'string'},
        {'id':'description', 'type':'text'},
        )
    title = ''
    description = ''

    __ac_permissions__=( ( 'Manage portal', () )
                       , ( 'Add portal content', () )
                       , ( 'Add portal folders', () )
                       , ( 'List portal members', () )
                       , ( 'Add writeups', () )
                       , ( 'Change writeups', () )
                       , ( 'View',  ( 'isEffective', ))
                       , ( 'Change nodes', () )
                       , ( 'Bless and curse writeups', () )
                       )

    def __init__( self, id, title='' ):
        PortalObjectBase.__init__( self, id, title )
        DefaultDublinCoreImpl.__init__( self )

    def isEffective( self, date ):
        """
            Override DefaultDublinCoreImpl's test, since we are always viewable.
        """
        return 1

    def reindexObject( self ):
        """
            Override DefaultDublinCoreImpl's method (so that we can play
            in 'editMetadata').
        """
        pass


Globals.InitializeClass(ZEverythingSite)


class PortalGenerator:

    klass = ZEverythingSite

    def setupTools(self, p):
        """Set up initial tools"""

        addCMFCoreTool = p.manage_addProduct['CMFCore'].manage_addTool
        addCMFCoreTool('CMF Skins Tool', None)
        addCMFCoreTool('CMF Types Tool', None)
        addCMFCoreTool('CMF Undo Tool', None)
        addCMFCoreTool('CMF Workflow Tool', None)

        addZETool = p.manage_addProduct['ZEverything'].manage_addTool
        addZETool('ZEverything Catalog', None)
        addZETool('ZEverything Actions Tool', None)
        addZETool('ZEverything Writeup Tool', None)
        addZETool('ZEverything Membership Tool', None)
        addZETool('ZEverything Member Data Tool', None)
        addZETool('ZEverything Registration Tool', None)
        addZETool('ZEverything URL Tool', None)
        addZETool('ZEverything Properties Tool', None)
        addZETool('ZEverything Metadata Tool', None)
        addZETool('ZEverything Node Tool', None)
        addZETool('ZEverything Message Tool', None)

    def setupMailHost(self, p):
        p.manage_addProduct['MailHost'].manage_addMailHost(
            'MailHost', smtp_host='localhost')

    def setupUserFolder(self, p):
        p.manage_addProduct['OFSP'].manage_addUserFolder()

    def setupCookieAuth(self, p):
        p.manage_addProduct['CMFCore'].manage_addCC(
            id='cookie_authentication')

    def setupMembersFolder(self, p):
        PortalFolder.manage_addPortalFolder(p, 'members')
        p.members.manage_addProduct['OFSP'].manage_addDTMLMethod(
            'index_html', '', '<dtml-return list_writeups>')

    def setupRoles(self, p):
        # Set up the suggested roles.
        p.__ac_roles__ = ('Member', 'Content Editor',)

    def setupPermissions(self, p):
        # Set up some suggested role to permission mappings.
        mp = p.manage_permission

        mp('Set own password',        ['Member','Manager',],    1)
        mp('Set own properties',      ['Member','Manager',],    1)
#        mp('List undoable changes',   ['Member','Manager',],    1)
        mp('Add portal content',      ['Owner','Manager',],     1)
        mp('Add portal folders',      ['Owner','Manager',],     1)
        mp('Bless and curse writeups',['Manager',],             1)
        mp('Add writeups',            ['Member','Manager',],    1)
        mp('Change writeups',         ['Owner','Manager','Content Editor',],1)
        mp('Change nodes',            ['Owner','Manager','Content Editor',],1)
        # Add some other permissions mappings that may be helpful.
        mp('Delete objects',          ['Owner','Manager',],     1)
        mp('Manage properties',       ['Owner','Manager',],     1)
        mp('Undo changes',            ['Owner','Manager',],     1)
        mp('View management screens', ['Owner','Manager',],     1)

    def setupDefaultSkins(self, p):
        from Products.CMFCore.DirectoryView import addDirectoryViews
        ps = getToolByName(p, 'portal_skins')
        addDirectoryViews(ps, 'skins', globals())
        addDirectoryViews(ps, 'skins', topic_globals)
        ps.manage_addProduct['OFSP'].manage_addFolder(id='custom')
        ps.addSkinSelection('Default',
            'custom, zpt_topic, zpt_content, zpt_generic,'
            + 'zpt_control, content, control, generic, Images', make_default=1)
        ps.addSkinSelection('print', 'print, custom, zpt_topic, zpt_content, zpt_generic,'
            + 'zpt_control, content, control, generic, Images')
        p.setupCurrentSkin()

    def setupTypes(self, p, initial_types=factory_type_information):
        tool = getToolByName(p, 'portal_types', None)
        if tool is None:
            return
        for t in initial_types:
            cfm = apply(ContentFactoryMetadata, (), t)
            tool._setObject(t['id'], cfm)
        
    def setupMimetypes(self, p):
        p.manage_addProduct['CMFCore'].manage_addRegistry()
        reg = p.content_type_registry

    def setupWorkflow(self, p):
        tool = getToolByName(p, 'portal_workflow', None)
        if tool is None:
            return
        from DefaultWorkflow import DefaultWorkflowDefinition
        from WriteupWorkflow import WriteupWorkflowDefinition
        tool._setObject('default_workflow', DefaultWorkflowDefinition('default_workflow'))
        tool._setObject('writeup_workflow', WriteupWorkflowDefinition('writeup_workflow'))
        tool.manage_changeWorkflows('default_workflow', {'chain_Writeup': 'writeup_workflow'})

    def setupNodebase(self, p):
        PortalFolder.manage_addPortalFolder(p, 'node')
        p.node.manage_permission('Add portal content', ['Member'], 1)
        from Globals import package_home
        file = os.path.join(package_home(globals()), 'default_node.zexp')
        p.node._importObjectFromFile(file)


    def setupMetadata(self, p):
        tool = getToolByName(p, 'portal_metadata', None)
        if tool is None:
            return
        spec = tool.getElementSpec('Subject')
        try:
            spec.addPolicy('Writeup')
        except:
            pass
        policy = spec.getPolicy('Writeup')
        policy.edit(is_required=1,supply_default=1,default_value=DEFAULT_SUBJECT, enforce_vocabulary=1, allowed_vocabulary=ALLOWED_SUBJECTS)

    def setupBlocks(self, p):
        from Globals import package_home
        file = os.path.join(package_home(globals()), 'blocks.zexp')
        p._importObjectFromFile(file)

    def setup(self, p, create_userfolder):
        self.setupTools(p)
        self.setupMailHost(p)
        if int(create_userfolder) != 0:
            self.setupUserFolder(p)
        self.setupCookieAuth(p)
        self.setupNodebase(p)
        self.setupMembersFolder(p)
        self.setupRoles(p)
        self.setupPermissions(p)
        self.setupMetadata(p)
        self.setupDefaultSkins(p)

        #   SkinnedFolders are only for customization;
        #   they aren't a default type.
        default_types = tuple( filter( lambda x: x['id'] != 'Skinned Folder'
                                     , factory_type_information ) )
        self.setupTypes(p, default_types )

        self.setupTypes(p, PortalFolder.factory_type_information)
        self.setupTypes(p, Topic.factory_type_information)
        self.setupMimetypes(p)
        self.setupWorkflow(p)
        self.setupBlocks(p)
        p.manage_addProduct['PageTemplates'].manage_addPageTemplate('index_html','','<span tal:replace="structure python:path(\'container/node/%s\' % \'1\')"></span>')

    def create(self, parent, id, create_userfolder):
        id = str(id)
        portal = self.klass(id=id)
        parent._setObject(id, portal)
        # Return the fully wrapped object.
        p = parent.this()._getOb(id)
        self.setup(p, create_userfolder)
        return p

    def setupDefaultProperties(self, p, title, description,
                               email_from_address, email_from_name,
                               validate_email,
                               ):
        p._setProperty('email_from_address', email_from_address, 'string')
        p._setProperty('email_from_name', email_from_name, 'string')
        p._setProperty('validate_email', validate_email and 1 or 0, 'boolean')
        p._setProperty('default_node', '1', 'string')
        p.title = title
        p.description = description


manage_addZEverythingForm = Globals.HTMLFile('dtml/addPortal', globals())
manage_addZEverythingForm.__name__ = 'addPortal'

def manage_addZEverything(self, id, title='Portal', description='',
                         create_userfolder=1,
                         email_from_address='postmaster@localhost',
                         email_from_name='Portal Administrator',
                         validate_email=0, RESPONSE=None):
    '''
    Adds a portal instance.
    '''
    gen = PortalGenerator()
    from string import strip
    id = strip(id)
    p = gen.create(self, id, create_userfolder)
    gen.setupDefaultProperties(p, title, description,
                               email_from_address, email_from_name,
                               validate_email)
    if RESPONSE is not None:
        RESPONSE.redirect(p.absolute_url() + '/manage_workspace')
