##############################################################################
#
# Copyright (c) 2002 Kelvin Mo
# 
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
# 
##############################################################################
__version__ = "0.5"

ADD_MEMBER_PERMISSION = 'Add portal member'

import Globals, Persistence, PersistentMapping, string, utils, re
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo, getSecurityManager
from Acquisition import Implicit, aq_base, aq_inner, aq_parent

from OFS.Image import Image

from Products.CMFCore.PortalFolder import PortalFolder
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.CMFCatalogAware import CMFCatalogAware
from Products.CMFCore.utils import _getViewFor, getToolByName
from zehtml.ZEHTMLParser import zehtml_parse_format
from DublinCore import DefaultDublinCoreImpl

from interfaces.Nodable import Nodable

factory_type_information = ( { 'id'             : 'Member Node'
                             , 'meta_type'      : 'Member Node'
                             , 'description'    : """\
A Member node is the home node for a member."""
                             , 'icon'           : 'node_icon.gif'
                             , 'product'        : 'ZEverything'
                             , 'factory'        : 'addUserNode'
                             , 'immediate_view' : ''
                             , 'actions'        :
                                ( { 'id'            : 'view'
                                  , 'name'          : 'View'
                                  , 'action'        : 'usernode_view'
                                  , 'permissions'   : (CMFCorePermissions.View,)
                                  },
                                  { 'id'            : 'scratchpad'
                                  , 'name'          : 'Scratchpad'
                                  , 'action'        : 'scratchpad'
                                  , 'permissions'   : (CMFCorePermissions.View,)
                                  , 'visible'       : 0
                                  },
                                )
                               }
                           ,
                           )

def addUserNode(self, id):
    o = UserNode(id)
    self._setObject(id,o)

class UserNode(Nodable, PortalContent):
    """ A user node """

    meta_type = 'Member Node'
    portal_type = 'Member Node'
    __implements__ = ( PortalContent.__implements__, )

    image = None
    cooked_text = bio = ''
    cooked_sp = sp = ''
    sp_private = 1

    review_state = 'published'    

    # Declarative security (replaces __ac_permissions__)
    security = ClassSecurityInfo()
    
    def __init__(self, id):
        self.id = id
        self.title = id

    security.declarePublic('edit')
    def edit(self, bio, image = None):
        self.bio, self.cooked_text=zehtml_parse_format(bio, 1)
        if image is not None:
            if self.image is None:
                self.image = Image('image', 'Image', '')
            self.image.manage_upload(image)

    security.declarePrivate( 'manage_beforeDelete' )
    def manage_beforeDelete(self, item, container):
        """
            Remove the contained items from the catalog.
        """
        if aq_base(bookmarks) is not aq_base(self):
            for obj in self.bookmarks.values():
                obj.__of__( self ).manage_beforeDelete(item, bookmarks)
                
    ### Content accessor methods
    security.declareProtected(CMFCorePermissions.View, 'SearchableText')
    def SearchableText(self):
        """ Used by the catalog for basic full text indexing """
        return self.Title()

    security.declareProtected(CMFCorePermissions.View, 'CookedBody')
    def CookedBody(self):
        return self.cooked_text

    security.declareProtected(CMFCorePermissions.View, 'EditableBody')
    def EditableBody(self):
        return self.bio

    security.declareProtected(CMFCorePermissions.View, 'Description')
    def Description(self):
        """ Dublin core description, also important for indexing """
        return "%s's home node" % self.title
    
    security.declareProtected(CMFCorePermissions.View, 'Format')
    def Format(self):
        return 'text/html'

    security.declareProtected(CMFCorePermissions.View, 'Type')
    def Type(self):
        return 'Node'

    security.declareProtected(CMFCorePermissions.View, 'Title')
    def Title(self):
        return self.title

    security.declareProtected(CMFCorePermissions.View, 'editScratchpad')
    def editScratchpad(self, text, private = 0, REQUEST=None):
        """ Edit the scratchpad"""
        self.sp_private = not not private
        self.sp, self.cooked_sp = zehtml_parse_format(text, 1)
        if REQUEST is not None:
            REQUEST.RESPONSE.redirect(self.absolute_url()+ '/scratchpad')

    security.declareProtected(CMFCorePermissions.View, 'getStorage')
    def getStorage(self):
        storage = getattr(self, 'storage', None)
        if not storage:
            self.createStorage()
        return storage

    security.declarePrivate('createStorage')
    def createStorage(self):
        self.storage = UserNodeStorage()
        self.storage.changeOwnership(getToolByName(self, 'portal_membership').getMemberById(self.id))
        self.manage_setLocalRoles(self.id, ['Owner',])
        return self.storage

InitializeClass(UserNode)

class UserNodeStorage(PortalFolder):
    """
        Storage for a user node.
    """

    security = ClassSecurityInfo()

    def __init__(self):
        self.id = 'storage'

    security.declareProtected(CMFCorePermissions.AccessContentsInformation, 'addBookmarkFolder')
    def addBookmarkFolder(self):
        self.manage_addPortalFolder('bookmarks', 'Bookmarks')

    security.declareProtected(CMFCorePermissions.View, 'index_html')
    def index_html(self, REQUEST):
        """
        A workaround that just shows the contents of the folder.
        """
        REQUEST.RESPONSE.redirect(self.absolute_url() + '/folder_contents')

    security.declareProtected(CMFCorePermissions.AddPortalContent, 'addBookmarkToFolder')
    def addBookmarkToFolder(self, target, id, title, remote_url):
        from Bookmark import addBookmark
        addBookmark(target, id, title, remote_url)

InitializeClass(UserNodeStorage)