##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
# Modified 2002 Kelvin Mo
# 
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
# 
##############################################################################
__version__ = "0.7"

ADD_CONTENT_PERMISSION = 'Add portal content'

import Globals, string, utils, re
from Globals import DTMLFile, InitializeClass
from AccessControl import ClassSecurityInfo
from AccessControl.User import nobody

from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.WorkflowCore import WorkflowAction
from Products.CMFCore.utils import keywordsplitter, getToolByName
from DublinCore import DefaultDublinCoreImpl
from utils import formatRFC822Headers, SimpleHTMLParser, bodyfinder, _dtmldir
from zehtml.ZEHTMLParser import zehtml_parse_format, zehtml_strip

from interfaces.Nodable import Nodable
from interfaces.Linkable import Linkable

from Products.PageTemplates.PageTemplate import PageTemplate, ModuleImporter

factory_type_information = ({'id'             : 'ZEverything Node',
                             'meta_type'      : 'ZEverything Node',
                             'description'    : """A ZEverything Node is the place to store ZEverything information.""",
                             'icon'           : 'node_icon.gif',
                             'product'        : '', # Hidden
                             'factory'        : 'addZENode',
                             'immediate_view' : 'node_view',
                             'actions'        : ({'id'            : 'view',
                                                  'name'          : 'View',
                                                  'action'        : 'node_view',
                                                  'permissions'   : (CMFCorePermissions.View, )},
                                                 {'id'            : 'edit',
                                                  'name'          : 'Edit',
                                                  'action'        : 'node_edit_form',
                                                  'permissions'   : ('Change nodes', )},
                                                 {'id'            : 'metadata',
                                                  'name'          : 'Metadata',
                                                  'action'        : 'metadata_edit_form',
                                                  'permissions'   : ('Change nodes', )},
                                                 { 'id'            : 'links',
                                                   'name'          : 'Links',
                                                   'action'        : 'node_links_form',
                                                   'permissions'   : ('Change nodes', )})
                            },)

def addZENode(self, id = '', title='', description='', text_format='zehtml', text=''):
    """ Add a ZEverything Node """
    if (id == ''):
        node_tool = getToolByName(self, 'portal_node')
        id = node_tool.getANodeId(self)
    o = ZENode(id, title, description, text_format, text)
    o.changeOwnership(nobody)
    self._setObject(id,o)
    return o

class ZENodeZPT(PageTemplate):
    """ A more friendly version of Page Template suitable for nodes"""

    ze_parent = None    

    def __init__(self, parent):
        if parent is not None:
            self.ze_parent = parent

    def pt_getContext(self):
        root = self.getPhysicalRoot()
        c = {'template': self,
             'here': self.ze_parent,
             'container': self.ze_parent.aq_inner.aq_parent,
             'nothing': None,
             'options': {},
             'root': root,
             'request': getattr(root, 'REQUEST', None),
             'modules': ModuleImporter
            }
        return c

class ZENode(Nodable, Linkable, PortalContent, DefaultDublinCoreImpl):
    """ A ZEverything Node"""

    __implements__ = (PortalContent.__implements__, DefaultDublinCoreImpl.__implements__)

    meta_type = 'ZEverything Node'
    portal_type = 'ZEverything Node'
    effective_date = expiration_date = None
    cooked_text = text = text_format = ''
    allow_writeup = 1
    allow_discussion = 1
    allow_softlinks = 1
    newsworthy = 0
    review_state = 'published'

    zehtml_level = 1                      # ZEHTML level    

    manage_options = (PortalContent.manage_options +
                      ({'label': 'Writeups',
                        'action': 'writeups/manage_workspace'},))

    # Declarative security (replaces __ac_permissions__)
    security = ClassSecurityInfo()
    
    def __init__(self, id, title='', description='', text_format='', text=''):
        DefaultDublinCoreImpl.__init__(self)
        self.id = id
        self.nodeid = self.id
        self.title = title
        self.description = description
        self.edit(text=text, text_format=text_format)
        self.setFormat(text_format)

    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'manage_edit')
    manage_edit = DTMLFile('zmi_editNode', _dtmldir)

    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'manage_editDocument')
    def manage_editDocument(self, text, text_format, zehtml_level, allow_discussion=0, allow_softlinks=0, newsworthy=0, file='', REQUEST=None):
        """ A ZMI (Zope Management Interface) level editing method """
        message = self.edit(text, text_format=text_format, zehtml_level=zehtml_level, allow_discussion=allow_discussion, allow_softlinks=allow_softlinks, newsworthy=newsworthy, file=file)
        if message is None:
            message = 'Document updated.'
        if REQUEST is not None:
            return self.manage_edit(manage_tabs_message=message)

    security.declareProtected('Change nodes', 'edit')
    def edit(self, text, file='', text_format='', zehtml_level=1, allow_discussion=1, allow_softlinks=1, newsworthy=0):
        """ Edit the Document - Parses headers and cooks the body"""
        if file and (type(file) is not type('')):
            t=file.read()
            if t != '':
                text = t
        self.zehtml_level = int(zehtml_level)
        if not text_format:
            text_format = self.text_format
        self.text_format = text_format
        self.allow_discussion= int(allow_discussion)
        self.allow_writeup = int(allow_discussion)
        self.allow_softlinks = not not allow_softlinks
        self.newsworthy = not not newsworthy       
        if text_format == 'zpt':
            self.text = text
            pt = ZENodeZPT(self)
            pt.expand = 0
            pt.pt_edit(text, 'text/html')
            self.text = pt._text
            if getattr(pt, '_v_warnings', None):
                raise 'ZPTError', ("<b>Warning:</b> <i>%s</i>" % join(self._v_warnings, '<br>'))           
        else:
            self.text, self.cooked_text = zehtml_parse_format(text, self.zehtml_level)
            #return None
        self.reindexObject()

    ### Content accessor methods
    security.declareProtected(CMFCorePermissions.View, 'SearchableText')
    def SearchableText(self):
        """ Used by the catalog for basic full text indexing """
        return "%s %s " % (self.Title(), zehtml_strip(self.EditableBody()))

    security.declareProtected(CMFCorePermissions.View, 'CookedBody')
    def CookedBody(self):
        """ Renders Page Templates if necessary """
        if self.text_format == 'zehtml':
            return self.cooked_text
        elif self.text_format == 'zpt':
            zpt = ZENodeZPT(self)
            zpt.expand = 0
            zpt.pt_edit(self.text, 'text/html')
            return zpt.pt_render(0)

    security.declareProtected(CMFCorePermissions.View, 'EditableBody')
    def EditableBody(self):
        """\
        The editable body of text.  This is the raw structured text, or
        in the case of HTML, what was between the <BODY> tags.
        """
        return self.text

    security.declareProtected(CMFCorePermissions.View, 'Description')
    def Description(self):
        """ Dublin core description, also important for indexing """
        return self.description
    
    security.declareProtected(CMFCorePermissions.View, 'Format')
    def Format(self):
        """ Returns a content-type style format of the underlying source """
        return 'text/html'

    def Type(self):
        return 'Node'
    
    security.declareProtected('Change nodes', 'setMetadata')
    def setMetadata(self, headers):
        headers['Format'] = self.Format()
        new_subject = keywordsplitter(headers)
        headers['Subject'] = new_subject or self.Subject()
        haveheader = headers.has_key
        for key, value in self.getMetadataHeaders():
            if key != 'Format' and not haveheader(key):
                headers[key] = value
        self._editMetadata(title=headers['Title'],
                          subject=headers['Subject'],
                          description=headers['Description'],
                          contributors=headers['Contributors'],
                          effective_date=headers['Effective_date'],
                          expiration_date=headers['Expiration_date'],
                          format=headers['Format'],
                          language=headers['Language'],
                          rights=headers['Rights'])
        self.reindexObject()

    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'setFormat')
    def setFormat(self, value):
        value = str(value)
        if value == 'text/html' or value == 'zehtml':
            self.text_format = 'zehtml'
        elif value == 'zpt':
            self.text_format = 'zpt'
    setFormat = WorkflowAction(setFormat)


InitializeClass(ZENode)
